;DABUG monitor ROM for C1P
;Supports full screen editor, BASIC keyword accelerator, and 48x16 (64x16) mode for OSI600D Superboard/C1P
;BASIC-IN-ROM mode
;
;annotated by Mark Spankus 8/01/2022

;$E0 = last keypress value when hook active
;$00E1-$00E6 contains code to store a character on the screen or load one for "full screen editor" mode.
 ;$E1 = A9  "LDA #$4F"
 ;$E2 = 4F
 ;$E3 = 8D  "STA $D304" or AD "LDA $D304"
 ;$E4 = current cursor screen addr lo
 ;$E5 = current cursor screen addr hi
 ;$E6 = 60   RTS
;$E7 = $FF means keyword expansion in progress
;$E8 = current keyword character position in BASIC token table
;$FA = screen width when hook installed 0=32 else 64
;$FB = $FF when hook installed
;$FC = various
;$FD = various
;$0200 = current cursor pos of input line on bottom row of screen $D3XX (or $D7xx for C2)
;$0201 = current character "under" cursor
;$0202 = temp storage for BASIC screen print $BF2D
;$0206 = screen print delay when not 0

*=$F800			;ROM starts at $F800
LF800               BIT $0203       ;test load flag
                    BPL LF808       ;branch if not set
                    JMP LFFBF       ;jmp to std load location
LF808               TYA
                    PHA             ;save Y & X
                    TXA
                    PHA
LF80C               JSR SF9DF       ;getkey or next character from keyword expansion
                    STA $E0
                    CMP #$15        ;CTRL+U
                    BNE LF829
                    JSR SF920
                    BCC LF81D
                    JSR SF938
LF81D               JSR $00E1
                    JSR SF94A
LF823               JSR SF938
                    JMP LF80C
LF829               CMP #$04        ;CTRL+D
                    BNE LF84C
                    JSR SF920
                    BCS LF80C
                    JSR $00E1       ; (store character on screen at cursor location)
                    LDA $E4         ;(E4,E5) = cursor address
                    LDY $FA         ;(FA) = 32 or 64 wide flag
                    BEQ LF841       ;0 = 32 wide
                    CLC
                    ADC #$40        ;add one row x64
                    JMP LF844
LF841               CLC
                    ADC #$20        ;add one row x32
LF844               STA $E4
                    BCC LF823
                    INC $E5         ;rollover to next page
                    BNE LF823
LF84C               CMP #$EC        ;CTRL+< key
                    BNE LF865
                    JSR SF920
                    BCS LF80C
                    JSR $00E1
                    LDX $E4
                    JSR SF993
                    STA $E4
                    BCS LF823
                    DEC $E5
                    BNE LF823
LF865               CMP #$EE        ;CTRL+> key
                    BNE LF877
                    JSR SF920
                    BCS LF88E
                    JSR $00E1
                    JSR SF963
                    JMP LF823
LF877               CMP #$01        ;CTRL+A key
                    BNE LF891
                    JSR SF920
                    BCS LF88E
                    JSR $00E1
                    STA $E0
                    JSR SF963
                    JSR SF938
                    JMP LF8F1
LF88E               JMP LF80C
LF891               CMP #$5F        ;_ / SHIFT+O
                    BNE LF8C6
                    PLA             ;"true backspace"
                    BNE LF89B
                    PHA
                    BEQ LF88E
LF89B               TAX
                    DEX
                    TXA
                    PHA
                    DEC $0E
                    LDA #$20
                    JSR SF9C9
                    LDY $FA         ;flag 0=32 else 64 wide
                    BEQ LF8B4
                    LDX $FD
                    JSR SF993
                    STA $FD
                    JMP LF8BD
LF8B4               LDX $0200
                    JSR SF993
                    STA $0200
LF8BD               TAX
                    LDA #$5F
                    STA $D300,X
                    JMP LF80C
LF8C6               CMP #$40         ;@ / SHIFT+P
                    BEQ LF8D9
                    CMP #$7F         ;RUBOUT key
                    BNE LF8D5
                    JSR SFCD5
LF8D1               LDA #$0D
                    STA $E0
LF8D5               CMP #$0D
                    BNE LF8E3
LF8D9               JSR SF9B6
LF8DC               PLA
                    TAX
                    PLA
                    TAY
                    LDA $E0
                    RTS
LF8E3               CMP #$02
                    BNE LF8F1
                    LDA $FA         ;flag 0=32 else 64 wide
                    EOR #$01
                    STA $FB         ;$FF when hook enabled
                    STA $FA         ;wide flag, 0=32
                    BPL LF8D1
LF8F1               LDY $FA         ;wide flag, 0=32
                    BEQ LF909
                    LDA #$65
                    STA $0200       ;current bottom screen cursor pos
                    LDA $FD
                    TAX
                    CMP #$7A
                    BEQ LF91A
                    JSR SF971
                    STA $FD
                    JMP LF8DC
LF909               LDA $0200       ;current bottom screen cursor pos
                    TAX
                    CMP #$7C
                    BEQ LF91A
                    JSR SF971
                    STA $0200       ;current bottom screen cursor pos
                    JMP LF8DC
LF91A               JSR SF94A
                    JMP LF8DC
SF920               LDA $E5         ;check cursor addr
                    CMP #$D3
                    BNE LF936
                    LDX $FA         ;flag 0=32 else 64 wide
                    BEQ LF92E
                    LDA #$4B
                    BNE LF930
LF92E               LDA #$65
LF930               CMP $E4
                    BNE LF936
                    SEC
                    RTS
LF936               CLC             ;return carry clear, addr OK
                    RTS
SF938               LDA #$AD        ;patch $00E3 code to LOAD from current cursor pos
                    STA $E3
                    JSR $00E3
                    STA $E2
                    LDA #$8D        ;restore $00E1 code to store at current cursor pos
                    STA $E3
                    LDA #$BB
                    JMP $00E3
SF94A               LDA $E4         ;Move current cursor pos up 1 line (32)
                    SEC
                    SBC #$20
                    STA $E4
                    BCS LF955
                    DEC $E5
LF955               LDX $FA         ;flag 0=32 else 64 wide
                    BEQ LF962
                    SEC             ;Move current cursor pos up 2nd half of 64 wide line
                    SBC #$20
                    STA $E4
                    BCS LF962
                    DEC $E5
LF962               RTS
SF963               INC $E4
                    LDX $E4
                    JSR SF971
                    STA $E4
                    BCC LF970
                    INC $E5
LF970               RTS
SF971               TXA
                    LDY $FA         ;flag 0=32 else 64 wide
                    BEQ LF980
                    ASL A
                    ASL A
                    CMP #$EC
                    BNE LF990
                    LDA #$10
                    BNE LF989
LF980               ASL A
                    ASL A
                    ASL A
                    CMP #$E8
                    BNE LF990
                    LDA #$08
LF989               STA $FC
                    TXA
                    CLC
                    ADC $FC
                    RTS
LF990               TXA
                    CLC
                    RTS
SF993               DEX
                    TXA
                    LDY $FA         ;flag 0=32 else 64 wide
                    BEQ LF9A3
                    ASL A
                    ASL A
                    CMP #$28
                    BNE LF9B3
                    LDA #$10
                    BNE LF9AC
LF9A3               ASL A
                    ASL A
                    ASL A
                    CMP #$20
                    BNE LF9B3
                    LDA #$08
LF9AC               STA $FC
                    TXA
                    SEC
                    SBC $FC
                    RTS
LF9B3               TXA
                    SEC
                    RTS
SF9B6               LDX #$06
LF9B8               LDA LF9D8,X
                    STA $E1,X
                    DEX
                    BPL LF9B8
                    LDX $FA         ;flag 0=32 else 64 wide
                    BEQ LF9C8
                    LDA #$4B
                    STA $E4
LF9C8               RTS
SF9C9               LDY $FA         ;flag 0=32 else 64 wide
                    BEQ LF9D1
                    LDY $FD
                    BNE LF9D4
LF9D1               LDY $0200
LF9D4               STA $D300,Y
                    RTS
LF9D8               LDA #$80        ;this bit of code gets copied to $00E1
                    STA $D365       ;and this address is the default cursor position
                    RTS             ;end of code
                    BRK
SF9DF               LDA $E7         ;is keyword expansion in progress?
                    BEQ LF9E8       ;0 = no
                    LDY $E8         ;get current index into keyword
                    JMP LFA1E
LF9E8               JSR LFD00       ;get key from keyboard
                    CMP #$1B        ;<ESC>
                    BEQ LF9F0
                    RTS
LF9F0               LDA #$12        ;start BASIC keyword lookup
                    JSR SF9C9       ;(write "=>" character to screen)
LF9F5               JSR LFD00       ;(get keypress)
                    CMP #$1B        ;<ESC>
                    BNE LFA04
                    LDA #$5F        ;_
                    JSR SF9C9
                    JMP LF9E8       ;(end keyword expansion)
LFA04               LDX #$44        ;convert key value to keyword index
LFA06               CMP LFA34,X     ;search table to find matching character
                    BEQ LFA10       ;branch when found
                    DEX
                    BPL LFA06
                    BMI LF9F5       ;lookup failed, start over
LFA10               INX             ;X=index of character in table (thus index of BASIC keyword)
                    LDY #$FF
LFA13               DEX
                    BEQ LFA1E       ;count down X until keyword found in BASIC table
LFA16               INY
                    LDA $A084,Y     ;count BASIC keywords (hi bit set = end of keyword)
                    BPL LFA16
                    BMI LFA13
LFA1E               INY             ;Y=index of BASIC keyword start
                    STY $E8
                    LDA $A084,Y     ;load keyword character from BASIC-IN-ROM keyword table
                    TAY
                    BMI LFA2D
                    LDA #$FF
LFA29               STA $E7         ;$E7=FF means keyword expansion in progress
                    TYA
                    RTS             ;return keyword character until done
LFA2D               AND #$7F
                    TAY
                    LDA #$00
                    BEQ LFA29
                    ; keyboard character to BASIC keyword position table $FA34-FA78
LFA34               .BYTE $3B,$31,$51,$36,$35,$5A,$59,$58
                    .BYTE $34,$0D,$00,$54,$33,$45,$0A,$4A
                    .BYTE $00,$00,$00,$4C,$53,$4F,$30,$20
                    .BYTE $4B,$7F,$2B,$00,$2F,$00,$00,$3F
                    .BYTE $32,$3E,$57,$00,$00,$00,$00,$00
                    .BYTE $3C,$00,$00,$00,$00,$44,$2D,$00
                    .BYTE $3A,$46,$00,$2C,$52,$4D,$2E,$56
                    .BYTE $43,$42,$4E,$50,$47,$49,$48,$41
                    .BYTE $55,$37,$39,$38,$00

LFA79               JSR LFD00        ;get key from keyboard
                    CMP #$11         ;^Q
                    BEQ LFA81
                    RTS

LFA81               TXA
                    PHA
                    TYA
                    PHA
                    LDA $00          ;test warm start vector for BASIC in ROM active
                    CMP #$4C
                    BNE LFA9A
                    LDA $01
                    CMP #$74
                    BNE LFA9A
                    LDA $02
                    CMP #$A2
                    BNE LFA9A
                    JSR SFAA1       ;if active go here
LFA9A               PLA
                    TAY
                    PLA
                    TAX
                    JMP ($0218)     ;(input vector)

SFAA1               JSR SF9B6
                    LDA $0200
                    STA $E4
                    LDA #$F8
                    STA $0219
                    LDA #$00
                    STA $0218       ;set input vector to F800
                    LDX $0200
                    LDA #$1D        ;put ^^ character on screen at current cursor
                    STA $D300,X
                    JMP LFBC8

LFABE               JSR SFAC4
                    JMP $BF2D

SFAC4               PHA
LFAC5               LDA #$FE       ;scan for CTRL+LEFT_SHIFT
                    STA $DF00      ;loop until not pressed
                    BIT $DF00
                    BVS LFAD9
                    STA $DF00
                    LDA $DF00
                    AND #$04
                    BEQ LFAC5
LFAD9               PLA
                    RTS

LFADB               .BYTE 'DAVID ANEAR 2/2/81',$FF
                    .BYTE $38,$EF,$18,$E8,$FC,$E8,$F5,$B7,$75
                    .BYTE $50,$3E,$B8,$8C,$05,$CB,$DB,$E4,$2C

LFB00               CMP #$7F         ;test for RUBOUT
                    BNE LFB09
                    JSR SFCD5
                    LDA #$0D
LFB09               STA $FE
                    PHA
                    LDA $FB          ;$FF when hook enabled
                    BMI LFB1B
                    STA $D800        ;toggle 48x16 screen mode
                    AND #$01
                    STA $FA          ;flag 0=32 else 64 wide
                    LDA #$FF
                    STA $FB          ;$FF when hook enabled
LFB1B               LDA $FA          ;flag 0=32 else 64 wide
                    BNE LFB23
                    PLA
                    JMP LFF69

LFB23               JSR SFAC4
                    TXA
                    PHA
                    TYA
                    PHA
                    LDA $FE          ;filter displayed characters in 48x16 mode to non-graphics <'{'
                    LDY $FF
                    LDX $FD
                    AND #$7F
                    CMP #$0D         ;^M  <CR>
                    BEQ LFB90
                    CMP #$08         ;^H  <BKSP>
                    BEQ LFB82
                    CMP #$0C         ;^L  <TAB>
                    BEQ LFB89
                    CMP #$0A         ;^J  <LF>
                    BEQ LFB9C
                    CMP #$20         ;<SPACE>
                    BMI LFB54
                    CMP #$7B         ;{
                    BPL LFB54
                    STA $D300,X
                    INX
                    CPX #$7B
                    BEQ LFB98
                    BNE LFB75
LFB54               LDY $D300,X
                    STY $FF
                    LDA #$5F         ;write underscore character to current cursor
                    STA $D300,X
                    STX $FD
                    PLA
                    TAY
                    PLA
                    TAX
                    PLA
                    JMP LFF6C

LFB68               TXA
                    AND #$3F
                    CMP #$0A
                    BNE LFB54
                    TXA
                    SBC #$10
                    TAX
                    BNE LFB54
LFB75               TXA
                    AND #$3F
                    CMP #$3B
                    BNE LFB54
                    TXA
                    ADC #$0F
                    TAX
                    BNE LFB54
LFB82               TYA
                    STA $D300,X
                    DEX
                    BCS LFB68
LFB89               TYA
                    STA $D300,X
                    INX
                    BCS LFB75
LFB90               TYA
                    STA $D300,X
                    LDX #$4B
                    BNE LFB54
LFB98               LDX #$4B
                    BNE LFBA0
LFB9C               TYA
                    STA $D300,X
LFBA0               STX $FC
                    LDA #$40
                    STA $F8
                    LDY #$00
                    STY $F6
                    LDX #$CF
LFBAC               INX
                    STX $F9
                    STX $F7
LFBB1               LDA ($F8),Y
                    STA ($F6),Y
                    INY
                    BEQ LFBAC
                    BPL LFBB1
                    CPX #$D3
                    BNE LFBB1
                    LDX $FC
                    JMP LFB54

LFBC3               .BYTE $29,$5D,$BB,$2C,$9B

LFBC8               LDA #$00
                    STA $FA          ;flag 0=32 else 64 wide
                    STA $FB          ;$FF when hook enabled
LFBCE               STA $021A
                    LDA #$4B
                    STA $FD
                    LDA #$FB
                    STA $021B
                    LDA #$20
                    STA $FF
                    RTS

LFBDF               JSR LFBC8
                    LDA #$09
                    JMP LFBCE
                    .BYTE $06

LFBE8               LDA $00	         ;check if warmstart vector set to BASIC-IN-ROM
                    CMP #$4C
                    BNE LFBFD
                    LDA $01
                    CMP #$74
                    BNE LFBFD
                    LDA $02
                    CMP #$A2
                    BNE LFBFD
                    JMP LFCD6		;jump if verified

LFBFD               JMP LFF31		;jump if not verified

  ;Initialize disk controller, read track 0 into memory at specified address
LFC00               JSR SFC0C
                    JMP ($00FD)      ;jump to address loaded from disk (usually $2200)
                    JSR SFC0C
                    JMP LFE00
SFC0C               LDY #$00         ;init disk controller
                    STY $C001        ;select DDRA.
                    STY $C000        ;0's in DDRA indicate input.
                    LDX #$04
                    STX $C001        ;select PORTA
                    STY $C003        ;select DDRB
                    DEY
                    STY $C002        ;1's in DDRB indicate output.
                    STX $C003        ;select PORT B
                    STY $C002        ;make all outputs high
                    LDA #$FB         ;set step towards 0
                    BNE LFC33
LFC2A               LDA #$02
                    BIT $C000
                    BEQ LFC4D        ;track 0 enabled?
                    LDA #$FF
LFC33               STA $C002        ;step off
                    JSR SFCA5        ; short delay
                    AND #$F7
                    STA $C002        ;step on
                    JSR SFCA5        ; short delay
                    ORA #$08
                    STA $C002        ;step off
                    LDX #$18         ;3ms delay @ 1MHz
                    JSR SFC91
                    BEQ LFC2A
LFC4D               LDX #$7F         ;load head
                    STX $C002
                    JSR SFC91        ;delay 158 ms @ 1MHz
LFC55               LDA $C000
                    BMI LFC55        ;wait for index start
LFC5A               LDA $C000
                    BPL LFC5A        ;wait for index end
                    LDA #$03
                    STA $C010        ;reset ACIA
                    LDA #$58
                    STA $C010        ;/1 RTS hi, no irq
                    JSR SFC9C
                    STA $FE          ;read start addr hi
                    TAX
                    JSR SFC9C
                    STA $FD          ;read start addr lo
                    JSR SFC9C
                    STA $FF          ;read num pages value
                    LDY #$00
LFC7B               JSR SFC9C        ;read the specified num pages
                    STA ($FD),Y      ;to specified address
                    INY
                    BNE LFC7B
                    INC $FE
                    DEC $FF
                    BNE LFC7B
                    STX $FE          ;restore start addr hi
                    LDA #$FF         ;disable drive
                    STA $C002
                    RTS
SFC91               LDY #$F8         ;delay for 1250 clock cycles * X
LFC93               DEY
                    BNE LFC93
                    EOR $FF,X        ;( extra instruction for time delay)
                    DEX
                    BNE SFC91
                    RTS
SFC9C               LDA $C010        ;wait for disk ACIA char
                    LSR A
                    BCC SFC9C
                    LDA $C011
SFCA5               RTS
SFCA6               LDA #$03         ;RESET serial ACIA @ $F000
                    STA $F000
                    LDA #$11         ; set 8N2 /16 RTS NOIRQ
                    STA $F000
                    RTS
SFCB1               PHA
LFCB2               LDA $F000        ;write to ACIA when possible
                    LSR A
                    LSR A
                    BCC LFCB2
                    PLA
                    STA $F001
                    RTS
SFCBE               EOR #$FF          ;Set inverted KB ROW (C1 patch for $FD00 keypoller)
                    STA $DF00
                    EOR #$FF
                    RTS
SFCC6               PHA               ;load KB inverted COL to X
                    JSR SFCCF
                    TAX
                    PLA
                    DEX
                    INX
                    RTS
SFCCF               LDA $DF00         ;load KB COL (C1 patch for $FD00 keypoller)
                    EOR #$FF
                    RTS

SFCD5               PHA               ;erase 2K screen (or 1K screen + 1K color?)
LFCD6               TXA
                    PHA
                    TYA
                    PHA
                    LDY #$00
                    LDA #$20
LFCDE               STA $D700,Y
                    STA $D600,Y
                    STA $D500,Y
                    STA $D400,Y
                    STA $D300,Y
                    STA $D200,Y
                    STA $D100,Y
                    STA $D000,Y
                    INY
                    BNE LFCDE
                    PLA
                    TAY
                    PLA
                    TAX
                    PLA
                    RTS
                   .BYTE $FF

;STD OSI C1P Keyboard decoder FD00-FDFF
;
;OSI keyboard decode table reference
;  Columns read from $DF00, Rows selected by writing to $DF00
;  C2/C4 series has noninverted values (bit set when key pressed)
;
;     7    6    5    4    3    2    1    0
; -----------------------------------------
; 7- 1/!  2/"  3/#  4/$  5/%  6/&  7/'
;
; 6- 8/(  9/)  0/@  :/*  -/=  rub
;
; 5- ./>   L    O    lf   cr
;
; 4-  W    E    R    T    Y    U    I
;
; 3-  S    D    F    G    H    J    K
;
; 2-  X    C    V    B    N    M   ,/<
;
; 1-  Q    A    Z   spc  //?  ;/+   P
;
; 0- rpt  ctl  esc            lsh  rsh  caps
;
;$0213 = temp storage for KB calculations
;$0214 = debounce counter
;$0215 = current key pressed
;$0216 = last key pressed

LFD00               TXA              ;entry point for get keypress
                    PHA
                    TYA
                    PHA
LFD04               LDA #$01         ;start with row 0
LFD06               JSR SFCBE        ;select row in A
                    JSR SFCC6        ;read selected row's columns to X
                    BNE LFD13        ;branch if any column set
LFD0E               ASL A
                    BNE LFD06
                    BEQ LFD66
LFD13               LSR A            ;test if row 0 selected
                    BCC LFD1F        ;branch if not
                    ROL A            ;restore A
                    CPX #$21         ; is ESC + CAPLOCK set?
                    BNE LFD0E
                    LDA #$1B         ;load ESC value
                    BNE LFD40        ;branch always
LFD1F               JSR SFDC8        ;return Y=leftmost bit set of A (0-7)
                    TYA
                    STA $0213        ;multiply A (ROW) by 7
                    ASL A
                    ASL A
                    ASL A            ;A*8
                    SEC
                    SBC $0213        ;-A
                    STA $0213        ;store row*7
                    TXA              ;X=current col
                    LSR A            ; /2
                    JSR SFDC8        ;Y = 0-7 current row
                    BNE LFD66        ; (multiple keys pressed? branch as if no keys pressed)
                    CLC
                    TYA              ; calculate row*7+col
                    ADC $0213
                    TAY
                    LDA LFDCF,Y       ;lookup key in table
LFD40               CMP $0215         ;same as last key pressed?
                    BNE LFD6B
                    DEC $0214         ;dec debounce counter
                    BEQ LFD75         ;branch when debounce done
                    LDY #$05          ;time delay 5ms @1MHz (5031 ticks)
LFD4C               LDX #$C8
LFD4E               DEX
                    BNE LFD4E
                    DEY
                    BNE LFD4C
                    BEQ LFD04        ;loop back to keyscan top
LFD56               CMP #$01         ;(r-shift?)
                    BEQ LFD8F        ;(incorrect R-Shift + key code returned $10 too high)
                    LDY #$00
                    CMP #$02         ;(l-shift?)
                    BEQ LFDA7
                    LDY #$C0
                    CMP #$20         ;(ctrl?)
                    BEQ LFDA7
LFD66               LDA #$00         ;(enter here when no keys pressed)
                    STA $0216
LFD6B               STA $0215        ;save current key pressed
                    LDA #$02
                    STA $0214        ;debounce value upon new key pressed
                    BNE LFD04        ;(loop back to key scan top)
LFD75               LDX #$96         ;auto-repeat loop delay value
                    CMP $0216        ;is this the same as last key?
                    BNE LFD7E
                    LDX #$14
LFD7E               STX $0214        ;(96=auto repeat delay, 14=new key delay)
                    STA $0216        ;store key as 'last' key value
                    LDA #$01         ;get KB row 0 columns
                    JSR SFCBE        ;select KB row
                    JSR SFCCF        ;get KB column
                    LSR A
                    BCC LFDC2        ;shift-lock pressed? branch if not
LFD8F               TAX
                    AND #$03         ;left or right shift pressed?
                    BEQ LFD9F        ;branch if not
                    LDY #$10         ;Y=amount to add to get shifted keycode
                    LDA $0215        ;is keycode high bit set? aka is it shiftable?
                    BPL LFDA7        ;branch if not
                    LDY #$F0         ;y=-$10 ('1'-$10 = '!')
                    BNE LFDA7
LFD9F               LDY #$00         ;y=amount to add for shifted value
                    CPX #$20         ;is CTRL pressed?
                    BNE LFDA7
                    LDY #$C0         ;'A'+$C0 = CTRL+A value
LFDA7               LDA $0215        ;return key value in $0215
                    AND #$7F         ;mask high bit
                    CMP #$20
                    BEQ LFDB7
                    STY $0213        ;Y=value to add to keycode
                    CLC
                    ADC $0213
LFDB7               STA $0213
                    PLA
                    TAY
                    PLA
                    TAX
                    LDA $0213
                    RTS
LFDC2               BNE LFD56        ;if any row 0 key pressed, branch
                    LDY #$20         ;amount to add to key to make it lower case
                    BNE LFDA7
SFDC8               LDY #$08         ;convert Y to rightmost column set
LFDCA               DEY
                    ASL A
                    BCC LFDCA
                    RTS              ;return Z flag set when only 1 bit set in A

                    ;keyboard key decode table
LFDCF               .BYTE $D0,$BB,$2F,$20,$5A,$41,$51 ;/ ZAQ,MNBVCXKJHGFDSIUYTREW
                    .BYTE $2C,$4D,$4E,$42,$56,$43,$58
                    .BYTE $4B,$4A,$48,$47,$46,$44,$53
                    .BYTE $49,$55,$59,$54,$52,$45,$57
                    .BYTE $00,$00,$0D,$0A,$4F,$4C,$2E
                    .BYTE $00,$FF,$2D,$BA,$30,$B9,$B8
                    .BYTE $B7,$B6,$B5,$B4,$B3,$B2,$B1

; 65V monitor for C1 FE00-FEFF
;$FC = current value
;$FD = not written but temporarily hex value copied to screen between addr & value
;$FE = current address low
;$FF = current address high
;
LFE00               LDX #$28         ;set stack top & clear decimal mode
                    TXS
                    CLD
                    NOP
                    NOP
                    NOP
                    NOP
                    NOP
                    NOP
                    NOP
                    NOP
                    LDX #$D4         ;(end page for screen clear  1K)
                    LDA #$D0         ;(start page for screen clear)
                    STA $FF
                    LDA #$00
                    STA $FE
                    STA $FB          ;load flag for monitor
                    TAY
                    LDA #$20
LFE1B               STA ($FE),Y      ;erase screen
                    INY
                    BNE LFE1B
                    INC $FF
                    CPX $FF
                    BNE LFE1B        ;loop until end page reached
                    STY $FF          ;set initial address to 0000
                    BEQ LFE43
LFE2A               JSR SFEE9        ;get key from input (KB or ACIA)
                    CMP #$2F         ;'/
                    BEQ LFE4F
                    CMP #$47
                    BEQ LFE4C
                    CMP #$4C
                    BEQ LFE7C        ;set load flag
                    JSR SFE93
                    BMI LFE2A
                    LDX #$02
                    JSR SFEDA
LFE43               LDA ($FE),Y
                    STA $FC
                    JSR SFEAC
                    BNE LFE2A
LFE4C               JMP ($00FE)      ;monitor "GO" command
LFE4F               JSR SFEE9
                    CMP #$2E         ;'.
                    BEQ LFE2A
                    CMP #$0D         ;<CR>
                    BNE LFE69
                    INC $FE
                    BNE LFE60
                    INC $FF
LFE60               LDY #$00
                    LDA ($FE),Y      ;read value from memory
                    STA $FC
                    JMP LFE77
LFE69               JSR SFE93
                    BMI LFE4F
                    LDX #$00
                    JSR SFEDA
                    LDA $FC
                    STA ($FE),Y
LFE77               JSR SFEAC
                    BNE LFE4F
LFE7C               STA $FB          ;(load flag for monitor)
                    BEQ LFE4F
LFE80               LDA $F000        ; wait for character from ACIA
                    LSR A
                    BCC LFE80
                    LDA $F001        ;return low 7 bits of value read
                    NOP
                    NOP
                    NOP
                    AND #$7F
                    RTS
LFE8F               BRK
                    BRK
                    BRK
                    BRK
SFE93               CMP #$30         ;convert HEX to binary nibble lo or $80 on err
                    BMI LFEA9
                    CMP #$3A         ;':
                    BMI LFEA6
                    CMP #$41         ;'A
                    BMI LFEA9
                    CMP #$47         ;'G
                    BPL LFEA9
                    SEC
                    SBC #$07
LFEA6               AND #$0F
                    RTS
LFEA9               LDA #$80
                    RTS
SFEAC               LDX #$03         ;display hex address to screen
                    LDY #$00         ;y=starting position offset on screen
LFEB0               LDA $FC,X        ;write hex byte to screen * 4 in reverse order
                    LSR A
                    LSR A
                    LSR A
                    LSR A
                    JSR SFECA        ;lo nibble hex out on screen (hi part 1st)
                    LDA $FC,X
                    JSR SFECA        ;lo nibble hex out on screen
                    DEX
                    BPL LFEB0
                    LDA #$20         ; (blank out 2 characters after address)
                    STA $D1CF        ;was $D0CA in syn600 (moved down on screen in this version)
                    STA $D1D0        ;was $D0CB
                    RTS
SFECA               AND #$0F         ;display low nibble of A as hex in next position on screen
                    ORA #$30
                    CMP #$3A
                    BMI LFED5
                    CLC
                    ADC #$07
LFED5               STA $D1CB,Y      ;address of screen memory for display; SYN600 was $D0C6,Y
                    INY
                    RTS
SFEDA               LDY #$04         ;shift in low nibble as address on screen
                    ASL A
                    ASL A
                    ASL A
                    ASL A            ;shift nibble down
LFEE0               ROL A
                    ROL $FC,X        ;shift in bits of current address
                    ROL $FD,X
                    DEY
                    BNE LFEE0        ;*4 bits
                    RTS
SFEE9               LDA $FB          ;(load flag for monitor when not 0)
                    BNE LFE80
                    JMP LFD00        ;get key from keyboard

        ;$FEF0-FEF9 default $0218+ I/O vectors initialization
LFEF0               .BYTE $BA,$FF    ; (input vector address)
LFEF2               .BYTE $69,$FF    ; (output vector address)
LFEF4               .BYTE $9B,$FF    ; (ctrl-c vector check address)
LFEE6               .BYTE $8B,$FF    ; (load vector address)
LFEF8               .BYTE $96,$FF    ; (save vector address)
                    .BYTE $30,$01    ; left over IRQ vector when located at $FF00
                    .BYTE $00,$FE    ; left over RESET vector when located at $FF00
                    .BYTE $C0,$01    ; left over NMI vector when located at $FF00

;    $FF00 ROM for C1 'D/C/W/M ?'
LFF00               CLD              ;location of RESET vector
                    LDX #$28
                    TXS
                    LDY #$0A
LFF06               LDA LFEF0-1,Y    ;initialize I/O vectors $0218-0221
                    STA $0217,Y
                    DEY
                    BNE LFF06
                    JSR SFCA6        ;reset ACIA @ $F000
                    STY $0212        ;CTRL-C flag !0 = ignore ctrl-c
                    STY $0203        ;LOAD flag
                    STY $0205        ;SAVE flag
                    STY $0206        ;time delay for CRT driver
                    LDA LFFE0        ;$FFE0 - cursor start
                    STA $0200
                    JSR SFCD5        ;(erase 2K screen)
                    BNE LFF35        ;(branch always)
SFF29               PHA
                    CMP #$7F
                    BNE LFF31
                    JMP LFBE8
LFF31               PLA
                    JMP LFABE
LFF35               LDA LFF5F,Y      ;display D/C/W/M ? prompt
                    BEQ LFF40
                    JSR $BF2D        ;BASIC screen print
                    INY
                    BNE LFF35
LFF40               JSR SFFBA
                    CMP #$4D
                    BNE LFF4A
                    JMP LFE00        ;jump to monitor
LFF4A               CMP #$57
                    BNE LFF51
                    JMP $0000        ;jump to warm start
LFF51               CMP #$43
                    BNE LFF58
                    JMP $BD11        ;jump to cold start
LFF58               CMP #$44
                    BNE LFF00
                    JMP LFC00        ;jump to disk boot
LFF5F               .BYTE 'D/C/W/M ?',0
LFF69               JSR SFF29        ;(default output vector location here)
LFF6C               PHA
                    LDA $0205        ;load save flag
                    BEQ LFF94        ;branch if save not enabled
                    PLA
                    JSR SFCB1        ;output to ACIA
                    CMP #$0D         ;was it <CR>?
                    BNE LFF95        ;branch if not
                    PHA
                    TXA
                    PHA
                    LDX #$0A         ;yes, output 10 NULs
                    LDA #$00
LFF81               JSR SFCB1        ;output to ACIA
                    DEX
                    BNE LFF81
                    PLA
                    TAX
                    PLA
                    RTS
LFF8B               PHA              ; (default load vector location here)
                    DEC $0203        ;set load flag
                    LDA #$00         ;load disable save flag
LFF91               STA $0205        ;store save flag
LFF94               PLA
LFF95               RTS              ;exit
LFF96               PHA              ; (default save vector location here)
                    LDA #$01         ; set save flag above @ $0205
                    BNE LFF91
                    LDA $0212        ; (default ctrl-c check vector here) 0212=ignore ctrl-c when not 0
                    BNE LFFB9
                    LDA #$FE
                    STA $DF00
                    BIT $DF00
                    BVS LFFB9
                    LDA #$FB
                    STA $DF00
                    BIT $DF00
                    BVS LFFB9
                    LDA #$03
                    JMP $A636
LFFB9               RTS
SFFBA               BIT $0203        ;(default input vector here) test load flag
                    BPL LFFD8
LFFBF               LDA #$FD         ;test space bar press
                    STA $DF00
                    LDA #$10
                    BIT $DF00
                    BEQ LFFD5        ;branch if pressed
                    LDA $F000        ;(READ char FROM ACIA)
                    LSR A            ;test input from ACIA ready?
                    BCC LFFBF
                    LDA $F001        ;unload ACIA
                    RTS              ;return value from ACIA
LFFD5               INC $0203
LFFD8               JMP LFA79        ; disable load flag
                    .BYTE $FF,$FF,$FF,$FF,$FF
LFFE0               .BYTE $65        ; $FFE0 - cursor start
LFFE1               .BYTE $17        ; $FFE1 - line len-1   (wrap position)
LFFE2               .BYTE $00        ; $FFE2 - 00 = 1K vid, 01=2K vid otherwise serial or other ROM
                    .BYTE $00,$03
                    .BYTE $FF,$9F
                    .BYTE $00,$03
                    .BYTE $FF,$9F
                    JMP ($0218)      ;input vector
                    JMP ($021A)      ;output vector
                    JMP ($021C)      ;ctrl-c vector check
                    JMP ($021E)      ;load vector
                    JMP ($0220)      ;save vector
                    .BYTE $30,$01    ;<IRQ>
                    .BYTE $00,$FF    ;<RESET>
                    .BYTE $C0,$01    ;<NMI>
